use crate::botv2::{
    domain::server_config::{
        change_enable_server::change_enable_server,
        check_if_server_enable_and_admin::check_if_server_enable_and_user_admin,
    },
    init::{Context, Error},
};
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter},
    CreateReply,
};
use tracing::instrument;

/// Initialize server config
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn init(
    ctx: Context<'_>,
    #[description = "Enable the bot to answer to the server"] enable: bool,
) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_server_enable_and_user_admin(guild.get(), ctx.author().id.get(), ctx.http())
        .await
    {
        Ok((ok, _)) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("You are not an admin")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };
    let answer = match change_enable_server(guild.get(), enable).await {
        Ok(_) => CreateEmbed::new()
            .title("Server config initialized")
            .color(colour::Color::DARK_GREEN),
        Err(_) => CreateEmbed::new()
            .title("Error initializing server config")
            .color(colour::Color::RED),
    };
    let builder = CreateReply::default()
        .embed(answer.footer(footer))
        .ephemeral(true);
    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
